(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     31265,        917]
NotebookOptionsPosition[     27912,        795]
NotebookOutlinePosition[     28329,        813]
CellTagsIndexPosition[     28286,        810]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " linear algebra works okay for finite fields of prime order (using modular \
arithmetic), but does not work well for general finite fields (using the \
FiniteFields package). See \
www.ma.rhul.ac.uk/~uvah099/Images/MathematicaLinearAlgebra.tiff."
}], "Text",
 CellChangeTimes->{{3.5415337428407803`*^9, 3.54153378460039*^9}, {
  3.5687247506916237`*^9, 3.568724854433477*^9}, {3.569186076682006*^9, 
  3.569186171721036*^9}, {3.5691862139488688`*^9, 3.5691862208148413`*^9}}],

Cell[TextData[{
 "To get started select \[OpenCurlyQuote]Evaluate Notebook\[CloseCurlyQuote] \
from the Evaluation menu and then try modifying the intructions for Example \
3.4. [",
 StyleBox["Added Question 8(b) on Sheet 3 on 8th February",
  FontWeight->"Bold"],
 ".]"
}], "Text",
 CellChangeTimes->{{3.5691862224399643`*^9, 3.5691863310570803`*^9}, {
  3.569348472155362*^9, 3.5693484914289083`*^9}}],

Cell[CellGroupData[{

Cell["Setup", "Section",
 CellChangeTimes->{{3.506621690456502*^9, 3.506621703324658*^9}, {
  3.50662221595536*^9, 3.506622221722838*^9}, {3.506625627159197*^9, 
  3.5066256348313427`*^9}, {3.538159323717499*^9, 3.5381593269726057`*^9}, {
  3.538173444651155*^9, 3.538173463608802*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Unprotect", "[", "Power", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"0", "^", "0"}], " ", ":=", "1"}]}], "Input",
 CellChangeTimes->{{3.506620982597632*^9, 3.506620993230542*^9}, {
   3.506621674195019*^9, 3.5066217135824738`*^9}, 3.5381591901516743`*^9, {
   3.538159337305724*^9, 3.538159359911775*^9}, {3.5687249620668373`*^9, 
   3.568724973344397*^9}, {3.568726403359886*^9, 3.568726403471961*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unprotect", "[", "E", "]"}], ";", " ", 
  RowBox[{"Unset", "[", "E", "]"}]}]], "Input",
 CellChangeTimes->{{3.538173736718554*^9, 3.5381737461716557`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Codewords u(f)", "Section",
 CellChangeTimes->{{3.569347063584331*^9, 3.5693470792069263`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"u", "[", 
   RowBox[{"f_", ",", "as_", ",", "x_", ",", "p_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Map", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Mod", "[", 
      RowBox[{
       RowBox[{"f", "/.", 
        RowBox[{"x", "\[Rule]", "#1"}]}], ",", "p"}], "]"}], "&"}], ",", 
    "as"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5693470802021313`*^9, 3.569347106176351*^9}, {
  3.569347138205139*^9, 3.569347191002206*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Key Equation for Reed\[Dash]Solomon Codes\
\>", "Section",
 CellChangeTimes->{{3.506620831320347*^9, 3.506620837691587*^9}, {
  3.538159374373851*^9, 3.538159376358542*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RowOfKeyEquationMatrix", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a_", ",", "v_"}], "}"}], ",", "k_", ",", "e_"}], "]"}], " ", ":=",
    " ", 
   RowBox[{"Module", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"firstPart", " ", "=", " ", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"a", "^", "j"}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "0", ",", 
            RowBox[{"k", "+", "e", "-", "1"}]}], "}"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"secondPart", " ", "=", " ", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "v"}], "*", 
           RowBox[{"a", "^", "j"}]}], ",", 
          RowBox[{"{", 
           RowBox[{"j", ",", "0", ",", "e"}], "}"}]}], "]"}]}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Join", "[", 
      RowBox[{"firstPart", ",", "secondPart"}], "]"}]}], "]"}]}], 
  "\[IndentingNewLine]"}]], "Input",
 CellChangeTimes->{{3.5066208391471577`*^9, 3.506620942026*^9}, {
  3.506621074915338*^9, 3.5066211317686377`*^9}, {3.506621411756743*^9, 
  3.506621414034277*^9}, {3.538159509362401*^9, 3.538159534410577*^9}, {
  3.538160985382421*^9, 3.538160986412899*^9}, {3.568726318202257*^9, 
  3.568726328358417*^9}, {3.5687264231463537`*^9, 3.568726434863443*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{"as_", ",", "vs_", ",", "k_"}], "]"}], " ", ":=", " ", 
  RowBox[{"Module", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"avPairs", " ", "=", " ", 
       RowBox[{"Thread", "[", 
        RowBox[{"{", 
         RowBox[{"as", ",", "vs"}], "}"}], "]"}]}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{"e", " ", "=", " ", 
       RowBox[{"Quotient", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Length", "[", "as", "]"}], " ", "-", "k"}], ",", "2"}], 
        "]"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"RowOfKeyEquationMatrix", "[", 
       RowBox[{"avPair", ",", "k", ",", "e"}], "]"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{"avPair", ",", " ", "avPairs"}], "}"}]}], "]"}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.506621113611044*^9, 3.506621211744458*^9}, {
  3.506621286030751*^9, 3.506621331764757*^9}, {3.506621366139573*^9, 
  3.5066213718606653`*^9}, {3.5066214340405827`*^9, 3.506621461679454*^9}, {
  3.5066215755861187`*^9, 3.506621576840662*^9}, {3.538160990925227*^9, 
  3.5381609954520397`*^9}, {3.5687263332227163`*^9, 3.568726337606494*^9}, {
  3.5687264407136374`*^9, 3.568726443777068*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Example 3.4: p=5, k=2, e=1 (so n=4)\
\>", "Section",
 CellChangeTimes->{{3.50662174702801*^9, 3.506621751563347*^9}, {
  3.506621787034992*^9, 3.5066218373674307`*^9}, {3.506621884532977*^9, 
  3.5066218847484503`*^9}, {3.538169964996883*^9, 3.538169967348678*^9}, {
  3.568724883149365*^9, 3.568724925435029*^9}}],

Cell[CellGroupData[{

Cell["General Key Equation Matrix for k=2, e=1", "Subsection",
 CellChangeTimes->{{3.5066227354688272`*^9, 3.5066227515668592`*^9}, {
  3.5381699701579313`*^9, 3.53816999274792*^9}, {3.538170229513204*^9, 
  3.5381702310709887`*^9}, {3.53817302619308*^9, 3.538173035006024*^9}, {
  3.538173096501616*^9, 3.538173104804438*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"v1", ",", "v2", ",", "v3", ",", "v4"}], "}"}], ",", "2"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506622097000791*^9, 3.506622109746628*^9}, {
  3.5066243717724247`*^9, 3.5066243758504467`*^9}, {3.5381701729205303`*^9, 
  3.538170207468577*^9}, {3.538173069693583*^9, 3.5381730730126457`*^9}, {
  3.568726340761281*^9, 3.568726340932433*^9}, {3.5687264490619793`*^9, 
  3.5687264491976423`*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "a1", 
      SuperscriptBox["a1", "2"], 
      RowBox[{"-", "v1"}], 
      RowBox[{
       RowBox[{"-", "a1"}], " ", "v1"}]},
     {"1", "a2", 
      SuperscriptBox["a2", "2"], 
      RowBox[{"-", "v2"}], 
      RowBox[{
       RowBox[{"-", "a2"}], " ", "v2"}]},
     {"1", "a3", 
      SuperscriptBox["a3", "2"], 
      RowBox[{"-", "v3"}], 
      RowBox[{
       RowBox[{"-", "a3"}], " ", "v3"}]},
     {"1", "a4", 
      SuperscriptBox["a4", "2"], 
      RowBox[{"-", "v4"}], 
      RowBox[{
       RowBox[{"-", "a4"}], " ", "v4"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.5381702022526207`*^9, 3.5381702088134193`*^9}, 
   3.538173168067143*^9, 3.538173220615201*^9, 3.568724873892168*^9, 
   3.5687263412749567`*^9, 3.56872640801215*^9, {3.5687264470670147`*^9, 
   3.568726449450653*^9}, 3.5688166207594833`*^9, 3.569186474704475*^9, 
   3.569346724218587*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Matrix for this code (recall that a1=0, a2=1, a3=2, a4=3)\
\>", "Subsection",
 CellChangeTimes->{{3.5066227354688272`*^9, 3.5066227515668592`*^9}, {
  3.5381699701579313`*^9, 3.53816999274792*^9}, {3.538170229513204*^9, 
  3.5381702310709887`*^9}, {3.53817302619308*^9, 3.538173035006024*^9}, {
  3.5381731078818808`*^9, 3.538173138880206*^9}, {3.568724930618219*^9, 
  3.568724931673036*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"as", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.53817326186839*^9, 3.538173268894857*^9}, {
  3.568724876989933*^9, 3.568724877982727*^9}, {3.568724934380155*^9, 
  3.5687249386990547`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{"as", ",", 
    RowBox[{"{", 
     RowBox[{"v1", ",", "v2", ",", "v3", ",", "v4"}], "}"}], ",", "2"}], 
   "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506622097000791*^9, 3.506622109746628*^9}, {
  3.5066243717724247`*^9, 3.5066243758504467`*^9}, {3.5381701729205303`*^9, 
  3.538170207468577*^9}, {3.538173069693583*^9, 3.5381730730126457`*^9}, {
  3.538173224919211*^9, 3.5381732711154203`*^9}, {3.568726363347045*^9, 
  3.568726363514752*^9}, {3.568726453646668*^9, 3.568726453773054*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", 
      RowBox[{"-", "v1"}], "0"},
     {"1", "1", "1", 
      RowBox[{"-", "v2"}], 
      RowBox[{"-", "v2"}]},
     {"1", "2", "4", 
      RowBox[{"-", "v3"}], 
      RowBox[{
       RowBox[{"-", "2"}], " ", "v3"}]},
     {"1", "3", "9", 
      RowBox[{"-", "v4"}], 
      RowBox[{
       RowBox[{"-", "3"}], " ", "v4"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.5381702022526207`*^9, 3.5381702088134193`*^9}, 
   3.538173176594496*^9, {3.538173244123825*^9, 3.5381732722416773`*^9}, 
   3.568724942854124*^9, 3.568724978791795*^9, {3.568726353749535*^9, 
   3.5687263637716913`*^9}, 3.5687264540744057`*^9, 3.568816620857308*^9, 
   3.569186474787169*^9, 3.5693467245262938`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(1) First example: receive (4,1,3,0)", "Subsection",
 CellChangeTimes->{{3.5066227354688272`*^9, 3.5066227515668592`*^9}, {
  3.5381699701579313`*^9, 3.538169971333582*^9}, {3.538173053934812*^9, 
  3.5381730585176277`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vs", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "3", ",", "0"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.506622097000791*^9, 3.506622109746628*^9}, {
  3.5066243717724247`*^9, 3.5066243758504467`*^9}, {3.538173352503503*^9, 
  3.538173359704134*^9}, {3.568724990176875*^9, 3.568724997088171*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  RowBox[{
   RowBox[{"KeyEquationMatrix", "[", 
    RowBox[{"as", ",", "vs", ",", "2"}], "]"}], ",", "5"}], "]"}]], "Input",
 CellChangeTimes->{{3.506622005233449*^9, 3.506622050662112*^9}, {
  3.506622117131192*^9, 3.5066221183963833`*^9}, {3.538173332668704*^9, 
  3.538173340504793*^9}, {3.56872636729844*^9, 3.568726367385147*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", 
      RowBox[{"-", "4"}], "0"},
     {"1", "1", "1", 
      RowBox[{"-", "1"}], 
      RowBox[{"-", "1"}]},
     {"1", "2", "4", 
      RowBox[{"-", "3"}], 
      RowBox[{"-", "6"}]},
     {"1", "3", "9", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$, 5]]]], "Output",
 CellChangeTimes->{{3.506622037190716*^9, 3.506622051114379*^9}, 
   3.506622118728693*^9, 3.5066221919821033`*^9, 3.506624376849358*^9, 
   3.5066252827814617`*^9, {3.506625685447885*^9, 3.506625694386524*^9}, {
   3.5066257755110283`*^9, 3.506625779881647*^9}, {3.50662582620146*^9, 
   3.506625855081551*^9}, 3.506625899257807*^9, {3.538173343058914*^9, 
   3.538173360903192*^9}, 3.568726198959141*^9, 3.568726367645296*^9, 
   3.568726409806231*^9, 3.56872645766785*^9, 3.568816620957609*^9, 
   3.569186474869375*^9, 3.5693467246164103`*^9, 3.569347015081585*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NullSpace", "[", 
  RowBox[{
   RowBox[{"KeyEquationMatrix", "[", 
    RowBox[{"as", ",", "vs", ",", "2"}], "]"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5066221478380547`*^9, 3.506622202030972*^9}, {
  3.538173364423856*^9, 3.5381733711513777`*^9}, {3.568726202010215*^9, 
  3.568726204682974*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "4", ",", "2", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "0", ",", "1", ",", "0"}], "}"}]}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.506622151994623*^9, 3.506622203618991*^9}, 
   3.506624377749385*^9, 3.50662528373447*^9, 3.506625968872151*^9, {
   3.538173365873138*^9, 3.538173371492979*^9}, 3.5687262050475607`*^9, 
   3.56872637100453*^9, 3.56872641169096*^9, 3.568816621041312*^9, 
   3.569186474911574*^9, 3.569346724672358*^9, 3.569347017141992*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolynomialQuotientRemainder", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"4", "x"}], "+", 
    RowBox[{"2", 
     RowBox[{"x", "^", "2"}]}]}], ",", "x", ",", "x", ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.568729102802765*^9, 3.568729122267906*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"2", " ", "x"}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.56872912270786*^9, 3.5688166211059933`*^9, 
  3.5691864749617863`*^9, 3.56934672470582*^9, 3.5693470191943703`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u", "[", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"2", "x"}]}], ",", "as", ",", "x", ",", "5"}], "]"}]], "Input",
 CellChangeTimes->{{3.569347119740501*^9, 3.5693471296541843`*^9}, {
  3.569347169676243*^9, 3.569347219215827*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "1", ",", "3", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.56934712174105*^9, 3.569347130396228*^9}, {
  3.569347162290731*^9, 3.569347224584545*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
(2) Second example: receive (4,0,3,0)\
\>", "Subsection",
 CellChangeTimes->{{3.5066227745258913`*^9, 3.506622777149612*^9}, {
  3.538176621661973*^9, 3.538176623796228*^9}, {3.538176984916679*^9, 
  3.538176986785345*^9}, {3.568726207731434*^9, 3.5687262092634277`*^9}, {
  3.568726413736499*^9, 3.5687264149899282`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vs2", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0", ",", "3", ",", "0"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.506622121058303*^9, 3.506622125936708*^9}, {
  3.506622603344966*^9, 3.5066226159204693`*^9}, {3.5066243796203823`*^9, 
  3.5066243836817293`*^9}, {3.506625240152461*^9, 3.5066252405421886`*^9}, {
  3.538176673099915*^9, 3.538176678468811*^9}, {3.568726227481841*^9, 
  3.568726235503536*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{"as", ",", "vs2", ",", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506621933852385*^9, 3.506621934409062*^9}, 
   3.5066226960696573`*^9, {3.5066227965527163`*^9, 3.5066227967580767`*^9}, {
   3.506625242886421*^9, 3.506625243237959*^9}, {3.53817668327013*^9, 
   3.538176688604004*^9}, {3.568726300254086*^9, 3.56872630675135*^9}, {
   3.568726373817919*^9, 3.56872637390591*^9}, {3.569347040275111*^9, 
   3.569347040402933*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", 
      RowBox[{"-", "4"}], "0"},
     {"1", "1", "1", "0", "0"},
     {"1", "2", "4", 
      RowBox[{"-", "3"}], 
      RowBox[{"-", "6"}]},
     {"1", "3", "9", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{3.569347040722261*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NullSpace", "[", 
  RowBox[{
   RowBox[{"KeyEquationMatrix", "[", 
    RowBox[{"as", ",", "vs2", ",", "2"}], "]"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5066228023580713`*^9, 3.5066228026601667`*^9}, {
  3.506625287371846*^9, 3.506625287722706*^9}, {3.538176692946516*^9, 
  3.538176698056685*^9}, {3.568726213866178*^9, 3.5687262156971684`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "2", ",", "4", ",", "1"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.506625255155829*^9, 3.5066252879696503`*^9}, 
   3.506625917653145*^9, 3.50662597418169*^9, {3.5381766951196957`*^9, 
   3.5381766984098167`*^9}, {3.568726216134451*^9, 3.568726237819121*^9}, 
   3.568726376157033*^9, 3.568726418807646*^9, 3.568726463366528*^9, 
   3.568816621303136*^9, 3.569186475107011*^9, 3.569346724888323*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolynomialQuotientRemainder", "[", 
  RowBox[{
   RowBox[{"1", "+", 
    RowBox[{"2", "x"}], "+", 
    RowBox[{"2", 
     RowBox[{"x", "^", "2"}]}]}], ",", 
   RowBox[{"4", "+", "x"}], ",", "x", ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.568729132235778*^9, 3.568729136412491*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"2", " ", "x"}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5687291372177477`*^9, 3.56881662137733*^9, 
  3.569186475156981*^9, 3.5693467249255543`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"u", "[", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"2", "x"}]}], ",", "as", ",", "x", ",", "5"}], "]"}]], "Input",
 CellChangeTimes->{{3.569347119740501*^9, 3.5693471296541843`*^9}, {
  3.569347169676243*^9, 3.569347219215827*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "1", ",", "3", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.56934712174105*^9, 3.569347130396228*^9}, {
  3.569347162290731*^9, 3.569347234844707*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["(3) Third example: receive (4,0,2,0)", "Subsection",
 CellChangeTimes->{{3.5066227745258913`*^9, 3.506622777149612*^9}, {
  3.5066238055445843`*^9, 3.506623806328072*^9}, {3.506624198691646*^9, 
  3.506624198785993*^9}, {3.538176989418124*^9, 3.538176993276185*^9}, {
  3.5687262391203623`*^9, 3.568726241169217*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"vs3", " ", "=", " ", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0", ",", "2", ",", "0"}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.506622121058303*^9, 3.506622125936708*^9}, {
  3.506622603344966*^9, 3.5066226159204693`*^9}, {3.506623816175557*^9, 
  3.506623816328046*^9}, {3.506623936874077*^9, 3.506623952688034*^9}, {
  3.506624389201729*^9, 3.506624393746715*^9}, {3.5066244872539043`*^9, 
  3.506624487468575*^9}, {3.506625297189769*^9, 3.506625299841179*^9}, {
  3.538177005196056*^9, 3.538177010330166*^9}, {3.568726243681172*^9, 
  3.56872625165593*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", 
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{"as", ",", "vs3", ",", "2"}], "]"}], "]"}]], "Input",
 CellChangeTimes->{{3.506621933852385*^9, 3.506621934409062*^9}, 
   3.5066226960696573`*^9, {3.5066227965527163`*^9, 3.5066227967580767`*^9}, {
   3.50662530293902*^9, 3.506625303266253*^9}, {3.5381770011785297`*^9, 
   3.538177013849576*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", 
      RowBox[{"-", "4"}], "0"},
     {"1", "1", "1", "0", "0"},
     {"1", "2", "4", 
      RowBox[{"-", "2"}], 
      RowBox[{"-", "4"}]},
     {"1", "3", "9", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.506622783839438*^9, 3.506622797066094*^9}, 
   3.5066238191550417`*^9, {3.506623938098723*^9, 3.5066239536723127`*^9}, 
   3.506624200581637*^9, 3.506624397141614*^9, {3.5066244894953136`*^9, 
   3.506624506896474*^9}, 3.506624537377472*^9, 3.50662530349347*^9, 
   3.5066259787777843`*^9, {3.5381770388827133`*^9, 3.538177045019628*^9}, 
   3.568726253253718*^9, 3.56872646593707*^9, 3.568816621489647*^9, 
   3.569186475259701*^9, 3.569346725089075*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NullSpace", "[", 
  RowBox[{
   RowBox[{"KeyEquationMatrix", "[", 
    RowBox[{"as", ",", "vs3", ",", "2"}], "]"}], ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5066228023580713`*^9, 3.5066228026601667`*^9}, {
  3.506625305468651*^9, 3.506625305842182*^9}, {3.538177047992236*^9, 
  3.538177049537362*^9}, {3.568726264805505*^9, 3.5687262668716927`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"4", ",", "3", ",", "3", ",", "1", ",", "0"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.506622803552869*^9, 3.5066238208919573`*^9, {3.506623938881118*^9, 
   3.506623954438314*^9}, 3.506624202313387*^9, 3.506624509033053*^9, 
   3.5066245392285*^9, 3.506625306611845*^9, 3.506625980294281*^9, 
   3.538177049825926*^9, {3.568726256640238*^9, 3.568726267271841*^9}, 
   3.568726467607349*^9, 3.568816621527073*^9, 3.5691864753164787`*^9, 
   3.569346725146809*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolynomialQuotientRemainder", "[", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"3", "x"}], "+", 
    RowBox[{"3", 
     RowBox[{"x", "^", "2"}]}]}], ",", "1", ",", "x", ",", 
   RowBox[{"Modulus", "\[Rule]", "5"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.5687291577222443`*^9, 3.568729168528294*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"4", "+", 
    RowBox[{"3", " ", "x"}], "+", 
    RowBox[{"3", " ", 
     SuperscriptBox["x", "2"]}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.568729164589806*^9, 3.568729168809799*^9}, 
   3.568816621579047*^9, 3.569186475373117*^9, 3.569346725253862*^9}]
}, Open  ]],

Cell["NB: not a codeword since k=2", "Text",
 CellChangeTimes->{{3.569347319013699*^9, 3.569347326081045*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Question 8(b) on Sheet 3", "Subsection",
 CellChangeTimes->{{3.569348300742869*^9, 3.569348313918824*^9}}],

Cell[BoxData[
 RowBox[{"M", " ", ":=", " ", 
  RowBox[{"KeyEquationMatrix", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"0", ",", "1", ",", "2", ",", "3", ",", "4"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "4", ",", "1", ",", "1", ",", "2"}], "}"}], ",", "3"}],
    "]"}]}]], "Input",
 CellChangeTimes->{{3.5693479150294847`*^9, 3.569347934139905*^9}, {
  3.569348256949533*^9, 3.5693482577463427`*^9}, {3.5693483248921957`*^9, 
  3.569348325150998*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MatrixForm", "[", "M", "]"}]], "Input",
 CellChangeTimes->{{3.569347351271162*^9, 3.5693473745449677`*^9}, {
  3.569347424775565*^9, 3.569347449300192*^9}, {3.569347710615281*^9, 
  3.569347710743181*^9}, {3.569347925284876*^9, 3.569347926459238*^9}}],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "0", "0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"1", "1", "1", "1", 
      RowBox[{"-", "4"}], 
      RowBox[{"-", "4"}]},
     {"1", "2", "4", "8", 
      RowBox[{"-", "1"}], 
      RowBox[{"-", "2"}]},
     {"1", "3", "9", "27", 
      RowBox[{"-", "1"}], 
      RowBox[{"-", "3"}]},
     {"1", "4", "16", "64", 
      RowBox[{"-", "2"}], 
      RowBox[{"-", "8"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.56934736100358*^9, 3.569347375248275*^9}, {
   3.569347425252219*^9, 3.569347449801921*^9}, 3.5693477110784483`*^9, {
   3.5693479268615522`*^9, 3.5693479360737467`*^9}, 3.5693482599354143`*^9, 
   3.569348326232757*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NullSpace", "[", 
  RowBox[{"M", ",", 
   RowBox[{"Modulus", "\[Rule]", "7"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.569348331902728*^9, 3.569348335158785*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"5", ",", "6", ",", "4", ",", "2", ",", "5", ",", "1"}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5693483355556803`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolynomialQuotientRemainder", "[", 
  RowBox[{
   RowBox[{"5", "+", 
    RowBox[{"6", "x"}], "+", 
    RowBox[{"4", 
     RowBox[{"x", "^", "2"}]}], "+", 
    RowBox[{"2", 
     RowBox[{"x", "^", "3"}]}]}], ",", 
   RowBox[{"5", "+", "x"}], ",", "x", ",", 
   RowBox[{"Modulus", "\[Rule]", "7"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.56934834042529*^9, 3.569348363605914*^9}, {
  3.5693484466109953`*^9, 3.569348451896421*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "+", "x", "+", 
    RowBox[{"2", " ", 
     SuperscriptBox["x", "2"]}]}], ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{{3.569348355213875*^9, 3.569348363972356*^9}, 
   3.569348453334284*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"Presentation",
WindowSize->{1106, 686},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"8.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (February 23, \
2011)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 542, 10, 102, "Text"],
Cell[1090, 32, 403, 9, 73, "Text"],
Cell[CellGroupData[{
Cell[1518, 45, 286, 4, 113, "Section"],
Cell[1807, 51, 455, 8, 78, "Input"],
Cell[2265, 61, 190, 4, 47, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2492, 70, 101, 1, 113, "Section"],
Cell[2596, 73, 459, 13, 47, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[3092, 91, 183, 4, 113, "Section"],
Cell[3278, 97, 1414, 36, 168, "Input"],
Cell[4695, 135, 1312, 31, 138, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6044, 171, 324, 6, 113, "Section"],
Cell[CellGroupData[{
Cell[6393, 181, 327, 4, 58, "Subsection"],
Cell[CellGroupData[{
Cell[6745, 189, 615, 13, 47, "Input"],
Cell[7363, 204, 1412, 40, 156, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8824, 250, 403, 7, 58, "Subsection"],
Cell[9230, 259, 312, 7, 47, "Input"],
Cell[CellGroupData[{
Cell[9567, 270, 585, 11, 47, "Input"],
Cell[10155, 283, 1236, 33, 140, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[11440, 322, 230, 3, 58, "Subsection"],
Cell[11673, 327, 361, 7, 47, "Input"],
Cell[CellGroupData[{
Cell[12059, 338, 379, 7, 47, "Input"],
Cell[12441, 347, 1362, 32, 138, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13840, 384, 371, 8, 47, "Input"],
Cell[14214, 394, 581, 12, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14832, 411, 315, 8, 47, "Input"],
Cell[15150, 421, 259, 6, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15446, 432, 259, 6, 47, "Input"],
Cell[15708, 440, 207, 4, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[15964, 450, 332, 6, 58, "Subsection"],
Cell[16299, 458, 464, 9, 47, "Input"],
Cell[CellGroupData[{
Cell[16788, 471, 531, 9, 47, "Input"],
Cell[17322, 482, 787, 23, 140, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[18146, 510, 420, 8, 47, "Input"],
Cell[18569, 520, 485, 9, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19091, 534, 349, 9, 47, "Input"],
Cell[19443, 545, 236, 6, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19716, 556, 259, 6, 47, "Input"],
Cell[19978, 564, 207, 4, 47, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20234, 574, 323, 4, 58, "Subsection"],
Cell[20560, 580, 603, 11, 47, "Input"],
Cell[CellGroupData[{
Cell[21188, 595, 389, 7, 47, "Input"],
Cell[21580, 604, 1223, 29, 140, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22840, 638, 420, 8, 47, "Input"],
Cell[23263, 648, 531, 11, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23831, 664, 327, 8, 47, "Input"],
Cell[24161, 674, 320, 8, 56, "Output"]
}, Open  ]],
Cell[24496, 685, 110, 1, 44, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[24643, 691, 112, 1, 58, "Subsection"],
Cell[24758, 694, 474, 11, 47, "Input"],
Cell[CellGroupData[{
Cell[25257, 709, 275, 4, 47, "Input"],
Cell[25535, 715, 1180, 33, 170, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26752, 753, 187, 4, 47, "Input"],
Cell[26942, 759, 184, 5, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27163, 769, 455, 12, 78, "Input"],
Cell[27621, 783, 251, 7, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
